<?php if ( ! defined('ABS_PATH')) exit('ABS_PATH is not loaded. Direct access is not allowed.');

/*
 * Copyright 2014 Osclass
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

    require_once osc_lib_path() . 'php-gettext/streams.php';
    require_once osc_lib_path() . 'php-gettext/gettext.php';

    class Translation {
        private $messages;
        private static $instance;

        public static function newInstance($install = false) {
            if(!self::$instance instanceof self) {
                self::$instance = new self($install);
            }
            return self::$instance;
        }

        function __construct($install = false) {
            if(!$install) {
                // get user/admin locale
                if( OC_ADMIN ) {
                    $locale = osc_current_admin_locale();
                } else {
                    $locale = osc_current_user_locale();
                }

                // load core
                $core_file = osc_translations_path() . $locale . '/core.mo';
                $this->_load($core_file, 'core');

                // load messages
                $messages_file = osc_themes_path() . osc_theme() . '/languages/' . $locale . '/messages.mo';
                if(!file_exists($messages_file)) {
                    $messages_file = osc_translations_path() . $locale . '/messages.mo';
                }
                $this->_load($messages_file, 'messages');

                // load theme
                $domain = osc_theme();
                $theme_file = osc_themes_path() . $domain . '/languages/' . $locale . '/theme.mo';
                if(!file_exists($theme_file)) {
                    if(!file_exists(osc_themes_path() . $domain)) {
                        $domain = 'modern';
                    }
                    $theme_file = osc_translations_path() . $locale . '/theme.mo';
                }
                $this->_load($theme_file, $domain);

                // load plugins
                $aPlugins = Plugins::listEnabled();
                foreach($aPlugins as $plugin) {
                    $domain = preg_replace('|/.*|', '', $plugin);
                    $plugin_file = osc_plugins_path() . $domain . '/languages/' . $locale . '/messages.mo';
                    if(file_exists($plugin_file) ) {
                        $this->_load($plugin_file, $domain);
                    }
                }
                // is rtl 
                $this->_isrtl($locale);
            } else {
                $core_file = osc_translations_path() . osc_current_admin_locale() . '/core.mo';
                $this->_load($core_file, 'core');
            }
        }

        function _get($domain) {
            if(!isset($this->messages[$domain])) {
                return false;
            }

            return $this->messages[$domain];
        }

        function _set($domain, $reader) {
            if(isset($messages[$domain])) {
               false;
            }

            $this->messages[$domain] = $reader;
            return true;
        }

        function _load($file, $domain) {
            if(!file_exists($file)) {
                return false;
            }

            $streamer = new FileReader($file);
            $reader = new gettext_reader($streamer);
            return $this->_set($domain, $reader);
        }

      /*******************************************************************************/
      /****************             is rtl_check         *****************************/
      /*******************************************************************************/

      function _isrtl($locale) {
          global $isrtl;
            if($locale=="fa_IR"){
               $isrtl= true;
                osc_add_hook("header",array('Translation','rtl_style'));
                osc_add_hook("admin_login_header",array('Translation', 'rtl_style_login_admin'));
                osc_add_filter('login_admin_title', array('Translation','fa_login_admin_title'));
                osc_add_filter('login_admin_url', array('Translation','fa_login_admin_url'));
                osc_add_filter('login_admin_image', array('Translation','fa_login_admin_image'));
                if(class_exists("Rewrite")){
                $location   = Rewrite::newInstance()->get_location();
                if(isset($location)){
                    $location = Params::getParam('page', false, false) ;
                }
                if(  $location=='categories' ) {
                    osc_unregister_script('jquery-nested');
                    osc_add_hook("admin_header",array('Translation','jquery_nested_rtl'));
                }
               }
            }
             $lang_check=substr($locale, 0, 2);
            if($lang_check=="fa"){
              osc_add_hook("index_admin_header", array('Translation','rtl_style_admin'));
            }
      }

      function jquery_nested_rtl(){
            osc_register_script('jquery-nested', osc_assets_url('js/jquery.ui.nestedSortable-rtl.js'), 'jquery');
            osc_enqueue_script('jquery-nested');
      }
      function fa_login_admin_title(){
            return"اواس کلاس فارسي";
       }
       function fa_login_admin_url(){
            return"http://faosclass.com/";
       }
       function fa_login_admin_image(){
         return osc_admin_base_url() . "images/faosclass-logo.gif";
       }
       function rtl_style_login_admin(){
          echo'<link type="text/css" href="style/rtl.css" media="screen" rel="stylesheet" />';
       }
       function rtl_style_admin(){
         osc_remove_style('admin-css');
         osc_unregister_script('admin-ui-osc');
         osc_unregister_script('admin-location');
         osc_register_script('admin-ui-osc',osc_current_admin_theme_js_url('ui-osc-rtl.js'), 'jquery');
         osc_register_script('admin-location', osc_current_admin_theme_js_url('location-rtl.js'), 'jquery');
         osc_enqueue_style('admin-css-rtl',  osc_current_admin_theme_styles_url('main-rtl.css'));
       }
       function rtl_style(){
            osc_enqueue_style('rtl', osc_current_web_theme_styles_url('rtl.css'));
       }
    }

?>